#ifndef GANGU_PYLIST_INSERTER_PARSER
#define GANGU_PYLIST_INSERTER_PARSER

namespace boost {
namespace spirit {


template<class ParserT>
struct pyobject_parser
	:unary< ParserT, parser< pyobject_parser<ParserT> > >
{
	typedef pyobject_parser<ParserT> self_t;
	typedef unary<ParserT,parser<self_t> > base_t;

	template<class ScannerT>
	struct result
	{
		typedef typename match_result<ScannerT, py::object>::type type;
	};

	pyobject_parser( ParserT const &p )
		:base_t(p)
	{
	}

	template<class ScannerT>
	typename parser_result<self_t,ScannerT>::type
	parse( ScannerT const &scan )const
	{
		typename parser_result<ParserT,ScannerT>::type subhit;
		ScannerT::iterator_t save = scan.first;

		subhit = this->subject().parse(scan);
		if( subhit && subhit.has_valid_attribute()){
			return scan.create_match( scan.first - save, py::object( subhit.value() ),
				scan.first, scan.first );
		}
		return scan.no_match();
	}
};

template<class ParserT, class Limit >
struct pylist_inserter_parser
	:unary< ParserT, parser< pylist_inserter_parser<ParserT,Limit> > >
{
	typedef pylist_inserter_parser<ParserT,Limit>	self_t;
	typedef unary<ParserT, parser<self_t> >	base_t;
	
	template<class ScannerT>
	struct result
	{
		typedef typename match_result<ScannerT, py::list>::type type;
	};

	pylist_inserter_parser( ParserT const &p, Limit const &l)
		:base_t( p ),limit(l)
	{
	}

	template<class ScannerT>
	typename parser_result<self_t,ScannerT>::type
	parse(ScannerT const &scan) const
	{
		typename parser_result<ParserT,ScannerT>::type subhit;
		ScannerT::iterator_t save = scan.first;
		py::list rlist;
		std::size_t n = limit;

		for( size_t i= 0; i < n ; ++i ){
			subhit = this->subject().parse(scan);
			if( subhit && subhit.has_valid_attribute()){
				rlist.append( subhit.value() );
			}
		}
		if( scan.first - save == 0 ){
			return scan.no_match();
		}
		return scan.create_match(
			scan.first - save, rlist, scan.first, scan.first );
	}

	Limit limit;
};

template<class ParserT> inline
pyobject_parser<ParserT> pyobject_p( ParserT const &p )
{
	return pyobject_parser<ParserT>(p);
}

template<class ParserT, class Exact> inline
pylist_inserter_parser<ParserT,Exact> pylist_p( ParserT const &p, Exact const &count )
{
	return pylist_inserter_parser<ParserT,Exact>(p, count);
}

}
}

#endif
















