#include "../include/config.hpp"
#include "../include/blender.hpp"
#include "../include/progress_parser.hpp"
#include "../include/vector_parser.hpp"
#include "../include/ObjectParser.hpp"

#include <boost/bimap/bimap.hpp>

namespace MQO {

using phoenix::bind;
using phoenix::arg1;

struct Face
{
	typedef boost::array< unsigned int, 4 > indices_t;
	typedef boost::optional< boost::array< float, 8 > >uvlist_t;
	typedef boost::optional< boost::array< unsigned int, 4 > > collist_t;

	indices_t indices;
	uvlist_t uvlist;
	collist_t collist;
	int matid;

	Face():matid( -1 ){}

	Face( indices_t i, int m, uvlist_t const &u, collist_t const &c )
		:indices(i),matid( m ), uvlist(u), collist(c)
	{
	}

	Face( Face const &x )
		:matid(x.matid), indices( x.indices), uvlist( x.uvlist ), collist( x.collist)
	{
	}
};

struct face_closure
	:closure< face_closure, Face >
{
	member1 mface;
};


bool ObjectParser::parseFace( scanner_t const &scan, py::object nmesh )const
{
	match<nil_t> firsthit = (str_p("face") >> int_p >> ch_p('{')).parse(scan);
	if( ! firsthit ){
		return false;
	}
	real_parser<float, real_parser_policies<float> > float_p;

	int number_of_vertex = 0;
	rule<scanner_t, face_closure::context_t> face_p;
	face_p =
		uint_p[ assign_a(number_of_vertex) ]
		>> str_p("V(")
		>> vector_p<4>( uint_p )
			[ bind(&Face::indices)(face_p.mface) = arg1 ]
		>> ch_p(')')
		>> !(
			str_p("M(")
			>> int_p[ bind(&Face::matid)(face_p.mface) = arg1 ]
			>> ch_p(')')
		)
		>> !(
			str_p("UV(")
			>> vector_p<8>( float_p )[ bind(&Face::uvlist)(face_p.mface) = arg1 ]
			>> ch_p(')')
		)
		>> !(
			str_p("COL(")
			>> vector_p<4>( uint_p )[ bind(&Face::collist)(face_p.mface) = arg1 ]
			>> ch_p(')')
		)
	;

	Face mface;
	int mesh_flag = 0;
	py::object create_face = Blender.NMesh().attr("Face");
	py::list verts( nmesh.attr("verts") );
	scanner_t::iterator_t save = scan.first;

	typedef boost::bimaps::bimap< int, int > bm_type;
	typedef bm_type::left_map::const_iterator left_const_iterator;
	bm_type matmap;

	while( !scan.at_end() && face_p[ assign_a(mface) ].parse(scan) ){
		if( number_of_vertex > 2 ){
			py::object bface = create_face();
			py::object append = bface.attr("append");
			int count = number_of_vertex;

			while( count-- ){
				append( verts.attr("__getitem__")(mface.indices[count]) );
			}

			if( mface.matid > -1 ){
				py::object material = materials.attr("__getitem__")(mface.matid);
				int matcount = py::extract<int>(
					nmesh.attr("getMaterials")().attr("count")( material ) );
				if( matcount == 0 ){
					if (nmesh.attr("materials").attr("__len__")() < 16){
						nmesh.attr("addMaterial")( material );
						matmap.insert( bm_type::value_type(mface.matid, (int)matmap.size()) );
					}
					else{
						std::cout << "too many materials. ignored \"" <<
								py::extract<char*>(material.attr("name")) << "\"" << std::endl;
					}
				}

				bm_type::left_const_iterator itr = matmap.left.find(mface.matid);

				if ( itr != matmap.left.end()){
					bface.attr("materialIndex") = itr->second;
					py::object mtexs = material.attr("getTextures")();
					if (mtexs && mtexs[0]){
						bface.attr("image") = mtexs[0].attr("tex").attr("image");
					}
				}
			}
			if( mface.uvlist ){
				py::list buv;
				mesh_flag |= 1;
				for( count = number_of_vertex * 2; count; count-=2 ){
					buv.append( py::make_tuple(
						mface.uvlist.get()[ count -2 ], 1.0 - mface.uvlist.get()[ count -1 ] ) );
				}
				bface.attr("uv") = buv;
			}
			if( mface.collist ){
				mesh_flag |= 2;
				py::object create_col = Blender.NMesh().attr("Col");
				py::list fcols;
				for( count = number_of_vertex; count; --count ){
					int col = mface.collist.get()[count-1];
					fcols.append( create_col(
						col & 255, col >> 8 & 255, col >> 16 & 255, col >> 24 & 255 ) );
				}
				bface.attr("col") = fcols;
			}
			nmesh.attr("faces").attr("append")( bface );
		}
		else if( number_of_vertex == 2 ){
			nmesh.attr("addEdge")(
				verts.attr("__getitem__")(mface.indices[1]),
				verts.attr("__getitem__")(mface.indices[0]) );
		}
	}

	ch_p('}').parse(scan);

	if( mesh_flag & 1 ){
		py::object mats = nmesh.attr("materials");
		for (int j=0; j<mats.attr("__len__")(); j++)
		{
			py::object mtex = mats[j].attr("getTextures")();
			for (int i=0; i<mtex.attr("__len__")(); i++)
			{
				if (mtex[i] && mtex[i].attr("tex").attr("getType")() == "Image"){
					nmesh.attr("hasFaceUV")(1);
				}
			}
		}
	}
	if( mesh_flag & 2 ){
		nmesh.attr("hasVertexColours")(1);
	}

	return scan.first != save;
}

}//MQO